from pathlib import Path

import pytest
from packaging.version import Version

from bumper.bump import BumpType, BumperFile, bump_ver


def test_empty_diff_feedback_single_file(dummy_repo: Path, capsys: pytest.CaptureFixture) -> None:
    pyproject = dummy_repo / "pyproject.toml"
    files = [
        BumperFile(file=pyproject, search='version = "{current_version}"'),
    ]

    bump_ver(
        current_version=Version("100.200.300"),
        files=files,
        bump_type=BumpType.MAJOR,
        dry_run=False,
    )

    captured = capsys.readouterr()
    assert "pyproject.toml - No changes" in captured.out


def test_empty_diff_feedback_multi_file(dummy_repo: Path, capsys: pytest.CaptureFixture) -> None:
    pyproject = dummy_repo / "pyproject.toml"
    readme = dummy_repo / "README.md"
    files = [
        BumperFile(file=pyproject, search='version = "{current_version}"'),
        BumperFile(file=readme, search="sco1-bumper/{current_version}"),
    ]

    bump_ver(
        current_version=Version("100.200.300"),
        files=files,
        bump_type=BumpType.MAJOR,
        dry_run=False,
    )

    captured = capsys.readouterr()
    assert "pyproject.toml - No changes" in captured.out
    assert "README.md - No changes" in captured.out


def test_bumped_feedback_single_file(dummy_repo: Path, capsys: pytest.CaptureFixture) -> None:
    pyproject = dummy_repo / "pyproject.toml"
    files = [
        BumperFile(file=pyproject, search='version = "{current_version}"'),
    ]

    bump_ver(
        current_version=Version("0.1.0"),
        files=files,
        bump_type=BumpType.MAJOR,
        dry_run=False,
    )

    captured = capsys.readouterr()
    assert "Bumped pyproject.toml" in captured.out


def test_bumped_feedback_multi_file(dummy_repo: Path, capsys: pytest.CaptureFixture) -> None:
    pyproject = dummy_repo / "pyproject.toml"
    readme = dummy_repo / "README.md"
    files = [
        BumperFile(file=pyproject, search='version = "{current_version}"'),
        BumperFile(file=readme, search="sco1-bumper/{current_version}"),
    ]

    bump_ver(
        current_version=Version("0.1.0"),
        files=files,
        bump_type=BumpType.MAJOR,
        dry_run=False,
    )

    captured = capsys.readouterr()
    assert "Bumped pyproject.toml" in captured.out
    assert "Bumped README.md" in captured.out



import tomllib
from pathlib import Path

import pytest
from packaging import version

from bumper.config import (
    BumperConfigError,
    BumperFile,
    PARSED_T,
    VersioningType,
    _validate_config,
    parse_config,
)
from tests import TEST_DATA_DIR

TOML_NO_TOOLS = """\
[project]
name = "sco1-bumper"
version = "0.1.0"
"""


def test_config_validation_no_tool_raises() -> None:
    cfg = tomllib.loads(TOML_NO_TOOLS)
    with pytest.raises(BumperConfigError, match="tools"):
        _validate_config(cfg)


TOML_NO_BUMPER = """\
[tool.black]
line-length = 100
"""


def test_config_validation_no_bumper_raises() -> None:
    cfg = tomllib.loads(TOML_NO_BUMPER)
    with pytest.raises(BumperConfigError, match="bumper configuration"):
        _validate_config(cfg)


TOML_MISSING_BUMPER_INFO = """\
[tool.bumper]
hello_world = "hi"
versioning_type = "semver"
"""


def test_config_validation_missing_bumper_info_raises() -> None:
    cfg = tomllib.loads(TOML_MISSING_BUMPER_INFO)
    with pytest.raises(BumperConfigError, match="current_version"):
        _validate_config(cfg)


TOML_MISSING_FILES = """\
[tool.bumper]
current_version = "0.1.0"
versioning_type = "semver"
"""


def test_config_validation_no_files_raises() -> None:
    cfg = tomllib.loads(TOML_MISSING_FILES)
    with pytest.raises(BumperConfigError, match="any file replacements"):
        _validate_config(cfg)


TOML_MISSING_FILE_INFO = """\
[tool.bumper]
current_version = "0.1.0"
versioning_type = "semver"

[[tool.bumper.files]]
file = "./pyproject.toml"
"""


def test_config_validation_missing_file_info_raises() -> None:
    cfg = tomllib.loads(TOML_MISSING_FILE_INFO)
    with pytest.raises(BumperConfigError, match="search"):
        _validate_config(cfg)


def test_nonexistent_config_raises() -> None:
    with pytest.raises(ValueError, match="does not exist"):
        parse_config(Path("ooga.booga"))


TRUTH_SINGLE_REPLACE_SEMVER = (
    version.Version("0.1.0"),
    VersioningType.SEMVER,
    [BumperFile(file=Path("./pyproject.toml"), search='version = "{current_version}"')],
)
TRUTH_SINGLE_REPLACE_CALVER = (
    version.Version("2025.1.0"),
    VersioningType.CALVER,
    [BumperFile(file=Path("./pyproject.toml"), search='version = "{current_version}"')],
)

TRUTH_MULTI_REPLACE_SEMVER = (
    version.Version("0.1.0"),
    VersioningType.SEMVER,
    [
        BumperFile(file=Path("./pyproject.toml"), search='version = "{current_version}"'),
        BumperFile(file=Path("./README.md"), search="sco1-bumper/{current_version}"),
        BumperFile(file=Path("./README.md"), search="rev: v{current_version}"),
    ],
)
TRUTH_MULTI_REPLACE_CALVER = (
    version.Version("2025.1.0"),
    VersioningType.CALVER,
    [
        BumperFile(file=Path("./pyproject.toml"), search='version = "{current_version}"'),
        BumperFile(file=Path("./README.md"), search="sco1-bumper/{current_version}"),
        BumperFile(file=Path("./README.md"), search="rev: v{current_version}"),
    ],
)

CONFIG_PARSER_TEST_CASES = (
    (TEST_DATA_DIR / "sample_config.toml", TRUTH_SINGLE_REPLACE_SEMVER),
    (TEST_DATA_DIR / "sample_pyproject.toml", TRUTH_SINGLE_REPLACE_SEMVER),
    (TEST_DATA_DIR / "sample_config_multi_replace.toml", TRUTH_MULTI_REPLACE_SEMVER),
    (TEST_DATA_DIR / "sample_pyproject_multi_replace.toml", TRUTH_MULTI_REPLACE_SEMVER),
    (TEST_DATA_DIR / "sample_config_calver.toml", TRUTH_SINGLE_REPLACE_CALVER),
    (TEST_DATA_DIR / "sample_pyproject_calver.toml", TRUTH_SINGLE_REPLACE_CALVER),
)


@pytest.mark.parametrize(("cfg_path", "truth_parsed"), CONFIG_PARSER_TEST_CASES)
def test_config_parse(cfg_path: Path, truth_parsed: PARSED_T) -> None:
    assert parse_config(cfg_path) == truth_parsed



from pathlib import Path

import pytest

from bumper.config import ExistingConfigError, VersioningType, write_default_config

STARTER_CONFIG_SEMVER = """\
[tool.bumper]
current_version = "0.1.0"
versioning_type = "semver"

[[tool.bumper.files]]
file = "./pyproject.toml"
search = 'version = "{current_version}"'
"""


def test_write_default_config_no_existing(tmp_path: Path) -> None:
    write_default_config(root_dir=tmp_path)

    cfg_path = tmp_path / ".bumper.toml"
    assert cfg_path.read_text() == STARTER_CONFIG_SEMVER


def test_write_default_config_existing_config_ignore_overwrites(tmp_path: Path) -> None:
    cfg_path = tmp_path / ".bumper.toml"
    cfg_path.touch()

    write_default_config(ignore_existing=True, root_dir=tmp_path)
    assert cfg_path.read_text() == STARTER_CONFIG_SEMVER


def test_write_default_config_existing_config_no_ignore_raises(tmp_path: Path) -> None:
    (tmp_path / ".bumper.toml").touch()
    with pytest.raises(ExistingConfigError, match="exists"):
        write_default_config(ignore_existing=False, root_dir=tmp_path)


STARTER_CONFIG_CALVER = """\
[tool.bumper]
current_version = "2025.1.0"
versioning_type = "calver"

[[tool.bumper.files]]
file = "./pyproject.toml"
search = 'version = "{current_version}"'
"""


def test_write_default_config_calver(tmp_path: Path) -> None:
    write_default_config(versioning_type=VersioningType.CALVER, root_dir=tmp_path)

    cfg_path = tmp_path / ".bumper.toml"
    assert cfg_path.read_text() == STARTER_CONFIG_CALVER



from pathlib import Path

import pytest

SAMPLE_README = """\
# bumper
[![PyPI - Python Version](https://some.url/sco1-bumper/0.1.0?logo=python)]

Automatically increment the project's version number.

```yaml
repos:
-   repo: https://github.com/sco1/brie-commit
    rev: v0.1.0
    hooks:
    -   id: brie-commit
```
"""

SAMPLE_PYPROJECT = """\
[project]
name = "sco1-bumper"
version = "0.1.0"
description = "Automatically increment the project's version number."

[build-system]
requires = ["hatchling"]
build-backend = "hatchling.build"
"""


@pytest.fixture
def dummy_repo(tmp_path: Path) -> Path:
    (tmp_path / "README.md").write_text(SAMPLE_README)
    (tmp_path / "pyproject.toml").write_text(SAMPLE_PYPROJECT)

    return tmp_path



from __future__ import annotations

import tomllib
import typing as t
from enum import StrEnum
from pathlib import Path

from packaging import version

BUMPER_REQUIRED_FIELDS = ("current_version", "versioning_type")
REPLACEMENT_REQUIRED_FIELDS = ("file", "search")


class BumperConfigError(Exception): ...  # noqa: D101


class BumperFile(t.NamedTuple):  # noqa: D101
    file: Path
    search: str

    @classmethod
    def from_toml(cls, file: str, search: str) -> BumperFile:  # noqa: D102
        return cls(file=Path(file), search=search)


class VersioningType(StrEnum):  # noqa: D101
    SEMVER = "semver"
    CALVER = "calver"


def _validate_config(cfg: dict) -> None:
    """
    Validate the provided parsed TOML output.

    The provided TOML is assumed to contain something like the following:

    ```toml
    [tool.bumper]
    current_version = "0.1.0"
    versioning_type = "semver"

    [[tool.bumper.files]]
    file = "./pyproject.toml"
    search = 'version = "{current_version}"'
    ```

    Raises `BumperConfigError` if any of the required information is missing.
    """
    if "tool" not in cfg:
        raise BumperConfigError("Configuration file does not declare any tools")

    if "bumper" not in cfg["tool"]:
        raise BumperConfigError("Configuration does not declare any bumper configuration")

    for rf in BUMPER_REQUIRED_FIELDS:
        if rf not in cfg["tool"]["bumper"]:
            raise BumperConfigError(f"Bumper tool declaration missing required field: '{rf}'")

    if "files" not in cfg["tool"]["bumper"]:
        raise BumperConfigError("Configuration does not declare any file replacements")

    for file in cfg["tool"]["bumper"]["files"]:
        for rf in REPLACEMENT_REQUIRED_FIELDS:
            if rf not in file:
                raise BumperConfigError(
                    f"File replacement declaration missing required field: '{rf}'"
                )


PARSED_T: t.TypeAlias = tuple[version.Version, VersioningType, list[BumperFile]]


def parse_config(cfg_path: Path) -> PARSED_T:
    """
    Parse the provided configuration file for its relevant information.

    Incoming information relevant to bumper is validated & extracted for downstream use.
    """
    if not cfg_path.exists():
        raise ValueError(f"Configuration file does not exist: '{cfg_path}'")

    with cfg_path.open("rb") as f:
        loaded = tomllib.load(f)

    _validate_config(loaded)
    current_version = version.parse(loaded["tool"]["bumper"]["current_version"])
    versioning_type = VersioningType(loaded["tool"]["bumper"]["versioning_type"])
    files = [BumperFile.from_toml(**f) for f in loaded["tool"]["bumper"]["files"]]

    return current_version, versioning_type, files


class ExistingConfigError(Exception): ...  # noqa: D101


STARTER_CONFIG_SEMVER = """\
[tool.bumper]
current_version = "0.1.0"
versioning_type = "semver"

[[tool.bumper.files]]
file = "./pyproject.toml"
search = 'version = "{current_version}"'
"""

STARTER_CONFIG_CALVER = """\
[tool.bumper]
current_version = "2025.1.0"
versioning_type = "calver"

[[tool.bumper.files]]
file = "./pyproject.toml"
search = 'version = "{current_version}"'
"""

CD = Path()


def write_default_config(
    versioning_type: VersioningType = VersioningType.SEMVER,
    ignore_existing: bool = False,
    root_dir: Path = CD,
) -> None:
    """
    Write a starter `.bumper.toml` configuration to the specified root directory.

    If `ignore_existing` is `True`, any existing `.bumper.toml` file will be overwritten; this
    action is not reversible. Otherwise, an exception will be raised.
    """
    cfg_path = root_dir / ".bumper.toml"
    if not ignore_existing:
        if cfg_path.exists():
            raise ExistingConfigError(f"Configuration file already exists: '{cfg_path.name}'")

    if versioning_type == VersioningType.SEMVER:
        cfg = STARTER_CONFIG_SEMVER
    elif versioning_type == VersioningType.CALVER:  # pragma: no branch
        cfg = STARTER_CONFIG_CALVER

    cfg_path.write_text(cfg)



from pathlib import Path

import pytest
from packaging.version import Version

from bumper.bump import BumpType, _build_new_version, _merge_bumpers, bump_ver
from bumper.config import BumperFile
from tests.conftest import SAMPLE_PYPROJECT, SAMPLE_README

VERSION_BUILD_TEST_CASES = (
    (Version("0.0.1"), BumpType.MAJOR, Version("1.0.0")),
    (Version("0.0.1"), BumpType.MINOR, Version("0.1.0")),
    (Version("0.0.1"), BumpType.PATCH, Version("0.0.2")),
    (Version("0.1.0"), BumpType.MAJOR, Version("1.0.0")),
    (Version("0.1.0"), BumpType.MINOR, Version("0.2.0")),
    (Version("0.1.0"), BumpType.PATCH, Version("0.1.1")),
    (Version("1.0.0"), BumpType.MAJOR, Version("2.0.0")),
    (Version("1.0.0"), BumpType.MINOR, Version("1.1.0")),
    (Version("1.0.0"), BumpType.PATCH, Version("1.0.1")),
    (Version("1.0"), BumpType.MAJOR, Version("2.0.0")),
    (Version("1.0"), BumpType.MINOR, Version("1.1.0")),
    (Version("1.0"), BumpType.PATCH, Version("1.0.1")),
)


@pytest.mark.parametrize(("current_version", "bump_type", "truth_out"), VERSION_BUILD_TEST_CASES)
def test_version_build(current_version: Version, bump_type: BumpType, truth_out: Version) -> None:
    assert _build_new_version(current_version, bump_type) == truth_out


def test_merge_bumpers() -> None:
    files = [
        BumperFile(file=Path("./pyproject.toml"), search='version = "{current_version}"'),
        BumperFile(file=Path("./README.md"), search="sco1-bumper/{current_version}"),
        BumperFile(file=Path("./README.md"), search="rev: v{current_version}"),
    ]

    truth_out = {
        Path("./pyproject.toml"): ['version = "{current_version}"'],
        Path("./README.md"): ["sco1-bumper/{current_version}", "rev: v{current_version}"],
    }

    assert _merge_bumpers(files) == truth_out


TRUTH_BUMPED_README = """\
# bumper
[![PyPI - Python Version](https://some.url/sco1-bumper/0.2.0?logo=python)]

Automatically increment the project's version number.

```yaml
repos:
-   repo: https://github.com/sco1/brie-commit
    rev: v0.2.0
    hooks:
    -   id: brie-commit
```
"""

TRUTH_BUMPED_PYPROJECT = """\
[project]
name = "sco1-bumper"
version = "0.2.0"
description = "Automatically increment the project's version number."

[build-system]
requires = ["hatchling"]
build-backend = "hatchling.build"
"""


def test_bump_ver_single_replace(dummy_repo: Path) -> None:
    pyproject = dummy_repo / "pyproject.toml"
    files = [
        BumperFile(file=pyproject, search='version = "{current_version}"'),
    ]

    bump_ver(current_version=Version("0.1.0"), files=files, bump_type=BumpType.MINOR, dry_run=False)

    assert pyproject.read_text() == TRUTH_BUMPED_PYPROJECT


TRUTH_SINGLE_DIFF = """\
--- pyproject.toml
+++
@@ -3 +3 @@
-version = "0.1.0"
+version = "0.2.0"
"""


def test_bump_ver_single_replace_dry_run(dummy_repo: Path, capsys: pytest.CaptureFixture) -> None:
    pyproject = dummy_repo / "pyproject.toml"
    files = [
        BumperFile(file=pyproject, search='version = "{current_version}"'),
    ]

    bump_ver(current_version=Version("0.1.0"), files=files, bump_type=BumpType.MINOR, dry_run=True)

    assert pyproject.read_text() == SAMPLE_PYPROJECT

    captured = capsys.readouterr()
    assert captured.out == TRUTH_SINGLE_DIFF


def test_bump_ver_multi_replace(dummy_repo: Path) -> None:
    pyproject = dummy_repo / "pyproject.toml"
    readme = dummy_repo / "README.md"
    files = [
        BumperFile(file=pyproject, search='version = "{current_version}"'),
        BumperFile(file=readme, search="sco1-bumper/{current_version}"),
        BumperFile(file=readme, search="rev: v{current_version}"),
    ]

    bump_ver(current_version=Version("0.1.0"), files=files, bump_type=BumpType.MINOR, dry_run=False)

    assert pyproject.read_text() == TRUTH_BUMPED_PYPROJECT
    assert readme.read_text() == TRUTH_BUMPED_README


TRUTH_MULTI_DIFF = """\
--- pyproject.toml
+++
@@ -3 +3 @@
-version = "0.1.0"
+version = "0.2.0"
--- README.md
+++
@@ -2 +2 @@
-[![PyPI - Python Version](https://some.url/sco1-bumper/0.1.0?logo=python)]
+[![PyPI - Python Version](https://some.url/sco1-bumper/0.2.0?logo=python)]
@@ -9 +9 @@
-    rev: v0.1.0
+    rev: v0.2.0
"""


def test_bump_ver_multi_replace_dry_run(dummy_repo: Path, capsys: pytest.CaptureFixture) -> None:
    pyproject = dummy_repo / "pyproject.toml"
    readme = dummy_repo / "README.md"
    files = [
        BumperFile(file=pyproject, search='version = "{current_version}"'),
        BumperFile(file=readme, search="sco1-bumper/{current_version}"),
        BumperFile(file=readme, search="rev: v{current_version}"),
    ]

    bump_ver(current_version=Version("0.1.0"), files=files, bump_type=BumpType.MINOR, dry_run=True)

    assert pyproject.read_text() == SAMPLE_PYPROJECT
    assert readme.read_text() == SAMPLE_README

    captured = capsys.readouterr()
    assert captured.out == TRUTH_MULTI_DIFF



import datetime as dt

import pytest
import time_machine
from packaging.version import Version

from bumper.bump import BumpType, _build_new_version

CALVER_TEST_CASES = (
    (Version("2025.1.0"), dt.date(year=2025, month=1, day=1), Version("2025.1.1")),
    (Version("2025.1"), dt.date(year=2025, month=1, day=1), Version("2025.1.1")),
    (Version("2025.10.0"), dt.date(year=2025, month=10, day=1), Version("2025.10.1")),
    (Version("2025.10"), dt.date(year=2025, month=10, day=1), Version("2025.10.1")),
    (Version("2025.1.0"), dt.date(year=2025, month=2, day=1), Version("2025.2.0")),
    (Version("2025.1.1"), dt.date(year=2025, month=2, day=1), Version("2025.2.0")),
    (Version("2025.10.0"), dt.date(year=2025, month=11, day=1), Version("2025.11.0")),
    (Version("2025.10.1"), dt.date(year=2025, month=11, day=1), Version("2025.11.0")),
)


@pytest.mark.parametrize(("current_version", "current_date", "truth_out"), CALVER_TEST_CASES)
def test_calver_version_build(
    current_version: Version,
    current_date: dt.date,
    truth_out: Version,
) -> None:
    with time_machine.travel(current_date, tick=False):
        new_ver = _build_new_version(current_version=current_version, bump_type=BumpType.DATE)

    assert new_ver == truth_out



import copy
import datetime as dt
import difflib
from collections import defaultdict
from enum import StrEnum
from pathlib import Path

from packaging import version

from bumper.config import BumperFile


class BumpType(StrEnum):  # noqa: D101
    # SemVer only
    MAJOR = "major"
    MINOR = "minor"
    PATCH = "patch"

    # CalVer only
    DATE = "date"


def _build_new_version(current_version: version.Version, bump_type: BumpType) -> version.Version:
    if bump_type == BumpType.DATE:
        utc_date = dt.datetime.now(dt.timezone.utc).date()
        version_year, version_month, version_micro = (
            current_version.major,
            current_version.minor,
            current_version.micro,
        )

        if (version_year == utc_date.year) and (version_month == utc_date.month):
            new_version = version.Version(f"{version_year}.{version_month:02}.{version_micro+1}")
        else:
            new_version = version.Version(f"{utc_date.year}.{utc_date.month:02}.0")
    else:
        major, minor, patch = current_version.major, current_version.minor, current_version.micro
        if bump_type is BumpType.MAJOR:
            new_version = version.Version(f"{major+1}.0.0")
        elif bump_type is BumpType.MINOR:
            new_version = version.Version(f"{major}.{minor+1}.0")
        elif bump_type is BumpType.PATCH:  # pragma: no branch
            new_version = version.Version(f"{major}.{minor}.{patch+1}")

    return new_version


def _merge_bumpers(files: list[BumperFile]) -> dict[Path, list[str]]:
    """Consolidate bump specifications per-file for downstream use."""
    file_operations = defaultdict(list)
    for b in files:
        file_operations[b.file].append(b.search)

    return file_operations


def bump_ver(
    current_version: version.Version, files: list[BumperFile], bump_type: BumpType, dry_run: bool
) -> None:
    """
    Bump the current version according to the provided rules in `files`.

    If `dry_run` is `True`, files will not be modified and a per-file diff will be printed to the
    terminal instead.

    If using CalVer (`bump_type` == `BumpType.DATE`), if the user's current UTC month is the same as
    the current project version, then the Micro component is incremented. Otherwise, the date
    components are bumped to the user's current UTC month and Micro reset to `0`.

    NOTE: Ensure that the bumper configuration file is included in the rules passed to `files`.

    NOTE: It is assumed that `bump_type` is appropriate for the project's configured versioning
    type.
    """
    next_version = _build_new_version(current_version, bump_type)
    file_operations = _merge_bumpers(files)  # Merge so we handle each file all at once

    for target_file, rules in file_operations.items():
        old = target_file.read_text()
        new = copy.copy(old)

        for r in rules:
            new = new.replace(
                r.replace("{current_version}", str(current_version)),
                r.replace("{current_version}", str(next_version)),
            )

        if new == old:
            print(f"{target_file.name} - No changes.")
            continue

        if dry_run:
            diff = difflib.unified_diff(
                old.splitlines(), new.splitlines(), fromfile=target_file.name, n=0, lineterm=""
            )
            # Strip trailing whitespace to make testing easier
            print("\n".join(line.rstrip() for line in diff))
        else:
            target_file.write_text(new)
            print(f"Bumped {target_file.name}")



import typing as t

import typer

from bumper import CONFIG_PRIORITY
from bumper.bump import BumpType, bump_ver
from bumper.config import (
    BumperConfigError,
    BumperFile,
    ExistingConfigError,
    VersioningType,
    parse_config,
    write_default_config,
)

bumper_cli = typer.Typer(add_completion=False)


def _abort_with_message(message: str, end: str = "\n") -> t.Never:
    print(message, end=end)
    raise typer.Abort()


@bumper_cli.command(name="bump")
def bump_ver_cmd(
    bump_by: BumpType,
    dry_run: bool = typer.Option(False, help="Preview the requested diff."),
) -> None:
    """
    Bump the requested version component.

    Allowable `BUMP_BY` values differ based on the project's specified versioning type: SemVer -
    (major, minor, patch), CalVer - (date)

    When using CalVer, if the user's current UTC month is the same as the current project version,
    then the Micro component is incremented. Otherwise, the date components are bumped to the user's
    current UTC month and Micro reset to `0`.

    If `dry_run` is `True`, the requested diff will be displayed in the terminal & no file
    modifications will take place.
    """
    for cfg_path in CONFIG_PRIORITY:
        if cfg_path.exists():
            break
    else:
        _abort_with_message("Configuration file could not be located.")

    try:
        current_version, versioning_type, files = parse_config(cfg_path)
    except BumperConfigError as e:
        _abort_with_message(str(e))

    # Check valid bump_by before we attempt to build a new version
    if versioning_type == VersioningType.SEMVER:
        if bump_by == BumpType.DATE:
            _abort_with_message("SemVer projects must bump by major, minor, or patch.")
    elif versioning_type == VersioningType.CALVER:
        if bump_by != BumpType.DATE:
            _abort_with_message("CalVer projects must bump by date.")

    # Add in the bump configuration so it gets updated as well
    files.append(BumperFile(file=cfg_path, search='current_version = "{current_version}"'))
    bump_ver(current_version=current_version, files=files, bump_type=bump_by, dry_run=dry_run)


@bumper_cli.command()
def init(
    versioning_type: VersioningType = VersioningType.SEMVER,
    ignore_existing: bool = typer.Option(False),
) -> None:
    """
    Generate a default bumper configuration file.

    If the `--ignore_existing` flag is set, any existing `.bumper.toml` file will be overwritten;
    this action is not reversible. Otherwise, the existing configuration will be preserved.
    """
    try:
        write_default_config(versioning_type=versioning_type, ignore_existing=ignore_existing)
    except ExistingConfigError as e:
        _abort_with_message(str(e))


if __name__ == "__main__":
    bumper_cli()
